@@ -35,6 +35,7 @@
 
 #define max(a, b) ((a) > (b) ? (a) : (b))
 #define min(a, b) ((a) < (b) ? (a) : (b))
+#define ARRAY_SIZE(x) (sizeof(x) / sizeof((x)[0]))
 
 enum process_field {
   process_pid = 0,
@@ -150,6 +151,7 @@ struct nvtop_interface {
   struct plot_window *plots;
   bool use_fahrenheit;
   bool show_plot;
+  bool plot_old_left_recent_right;
   double encode_decode_hide_time;
   struct retained_data past_data;
 };
@@ -166,13 +168,13 @@ enum device_field {
 
 static unsigned int sizeof_device_field[] = {
     [device_name] = 11,  [device_fan_speed] = 8, [device_temperature] = 10,
-    [device_power] = 15, [device_clock] = 11,    [device_pcie] = 44,
+    [device_power] = 15, [device_clock] = 11,    [device_pcie] = 46,
 };
 
 static unsigned int sizeof_process_field[] = {
-    [process_pid] = 5,       [process_user] = 4,          [process_gpu_id] = 3,
+    [process_pid] = 7,       [process_user] = 4,          [process_gpu_id] = 3,
     [process_type] = 7,
-    [process_memory] = 13, // 8 for mem 5 for %
+    [process_memory] = 14, // 9 for mem 5 for %
     [process_cpu_usage] = 6, [process_cpu_mem_usage] = 9, [process_command] = 0,
 };
 
@@ -431,7 +433,7 @@ static void initialize_all_windows(struct nvtop_interface *dwin) {
   struct window_position *plot_positions = NULL;
 
   enum plot_type plot_type;
-  compute_sizes_from_layout(dwin->show_plot, true, true, dwin->num_devices, 2,
+  compute_sizes_from_layout(dwin->show_plot, true, true, num_devices, 2,
                             3, device_length(), rows - 1, cols,
                             device_positions, &process_position,
                             &dwin->num_plots, &plot_positions, &plot_type);
@@ -510,6 +512,7 @@ static void initialize_colors(void) {
 struct nvtop_interface *
 initialize_curses(unsigned int num_devices, unsigned int biggest_device_name,
                   bool use_color, bool use_fahrenheit, bool show_per_gpu_plot,
+                  bool plot_old_left_recent_right,
                   double encode_decode_hide_time, unsigned collect_interval) {
   struct nvtop_interface *interface = calloc(1, sizeof(*interface));
   interface->devices_win = calloc(num_devices, sizeof(*interface->devices_win));
@@ -535,6 +538,7 @@ initialize_curses(unsigned int num_devices, unsigned int biggest_device_name,
   interface->process.sort_criterion = process_memory;
   interface->process.sort_asc = false;
   interface->process.size_processes_buffer = 50;
+  interface->plot_old_left_recent_right = plot_old_left_recent_right;
   interface->process.all_process =
       malloc(interface->process.size_processes_buffer *
              sizeof(*interface->process.all_process));
@@ -606,7 +610,7 @@ static void draw_bare_percentage(WINDOW *win, const char *prelude,
   wnoutrefresh(win);
 }
 
-static const char *memory_prefix[] = {"B", "k", "M", "G", "T", "P"};
+static const char *memory_prefix[] = {" B", "Ki", "Mi", "Gi", "Ti", "Pi"};
 
 static void draw_temp_color(WINDOW *win, unsigned int temp,
                             unsigned int temp_slowdown, bool celsius) {
@@ -637,7 +641,7 @@ static void print_pcie_at_scale(WINDOW *win, unsigned int value) {
   int prefix_off;
   double val_d = value;
   for (prefix_off = 1; prefix_off < 5 && val_d >= 1000.; ++prefix_off) {
-    val_d = val_d / 1000.;
+    val_d = val_d / 1024.;
   }
   if (val_d >= 100.) {
     wprintw(win, "%.1f", val_d);
@@ -784,10 +788,10 @@ static void draw_devices(struct device_info *dev_info,
       double used_mem = dinfo->used_memory;
       size_t prefix_off;
       for (prefix_off = 0; prefix_off < 5 && total_mem >= 1000; ++prefix_off) {
-        total_mem /= 1000;
-        used_mem /= 1000;
+        total_mem /= 1024;
+        used_mem /= 1024;
       }
-      snprintf(buff, 1024, "%.1f%s/%.1f%s", used_mem, memory_prefix[prefix_off],
+      snprintf(buff, 1024, "%.3f%s/%.3f%s", used_mem, memory_prefix[prefix_off],
                total_mem, memory_prefix[prefix_off]);
       draw_bare_percentage(mem_util_win, "MEM",
                            (unsigned int)(100. * dinfo->used_memory /
@@ -845,6 +849,12 @@ static void draw_devices(struct device_info *dev_info,
         IS_VALID(power_draw_max_valid, dinfo->valid))
       mvwprintw(dev->power_info, 0, 0, "POW %3u / %3u W",
                 dinfo->power_draw / 1000, dinfo->power_draw_max / 1000);
+    else if (IS_VALID(power_draw_valid, dinfo->valid) &&
+             !IS_VALID(power_draw_max_valid, dinfo->valid))
+      mvwprintw(dev->power_info, 0, 0, "POW %3u W", dinfo->power_draw / 1000);
+    else if (!IS_VALID(power_draw_valid, dinfo->valid) &&
+             IS_VALID(power_draw_max_valid, dinfo->valid))
+      mvwprintw(dev->power_info, 0, 0, "POW N/A / %3u W", dinfo->power_draw_max / 1000);
     else
       mvwprintw(dev->power_info, 0, 0, "POW N/A W");
     mvwchgat(dev->power_info, 0, 0, 3, 0, cyan_color, NULL);
@@ -1218,8 +1228,8 @@ static void print_processes_on_screen(unsigned int num_process,
                           process_buffer_line_size - printed, " %*s ",
                           sizeof_process_field[process_type], "Compute");
     }
-    snprintf(memory, 9, "%6lluMB", proc[i].used_memory / 1048576);
-    snprintf(memory + 8, sizeof_process_field[process_memory] - 7, " %3.0f%%",
+    snprintf(memory, 10, "%6uMiB", (unsigned)(proc[i].used_memory / 1048576));
+    snprintf(memory + 9, sizeof_process_field[process_memory] - 7, " %3.0f%%",
              proc[i].mem_percentage);
     printed += snprintf(&process_print_buffer[printed],
                         process_buffer_line_size - printed, "%*s ",
@@ -1229,7 +1239,7 @@ static void print_processes_on_screen(unsigned int num_process,
     printed += snprintf(&process_print_buffer[printed],
                         process_buffer_line_size - printed, "%*s ",
                         sizeof_process_field[process_cpu_usage], cpu_percent);
-    snprintf(cpu_mem, sizeof_process_field[process_cpu_mem_usage] + 1, "%zuMB",
+    snprintf(cpu_mem, sizeof_process_field[process_cpu_mem_usage] + 1, "%zuMiB",
              proc[i].cpu_memory / 1048576);
     printed += snprintf(&process_print_buffer[printed],
                         process_buffer_line_size - printed, "%*s ",
@@ -1279,22 +1289,75 @@ static void draw_processes(struct device_info *dev_info,
   print_processes_on_screen(total_processes, &interface->process);
 }
 
-static const char *signalsName[] = {
-    "Cancel",  "SIGABRT", "SIGALRM",   "SIGBUS",  "SIGCHLD", "SIGCONT",
-    "SIGFPE",  "SIGHUP",  "SIGILL",    "SIGINT",  "SIGKILL", "SIGPIPE",
-    "SIGQUIT", "SIGSEGV", "SIGSTOP",   "SIGTERM", "SIGTSTP", "SIGTTIN",
-    "SIGTTOU", "SIGUSR1", "SIGUSR2",   "SIGPOLL", "SIGPROF", "SIGSYS",
-    "SIGTRAP", "SIGURG",  "SIGVTALRM", "SIGXCPU", "SIGXFSZ",
+static const char *signalNames[] = {
+  "Cancel",
+  "SIGHUP",
+  "SIGINT",
+  "SIGQUIT",
+  "SIGILL",
+  "SIGTRAP",
+  "SIGABRT",
+  "SIGBUS",
+  "SIGFPE",
+  "SIGKILL",
+  "SIGUSR1",
+  "SIGSEGV",
+  "SIGUSR2",
+  "SIGPIPE",
+  "SIGALRM",
+  "SIGTERM",
+  "SIGCHLD",
+  "SIGCONT",
+  "SIGSTOP",
+  "SIGTSTP",
+  "SIGTTIN",
+  "SIGTTOU",
+  "SIGURG",
+  "SIGXCPU",
+  "SIGXFSZ",
+  "SIGVTALRM",
+  "SIGPROF",
+  "SIGWINCH",
+  "SIGIO",
+  "SIGPWR",
+  "SIGSYS",
 };
 
-static const int signalsValues[] = {
-    SIGABRT, SIGALRM, SIGBUS,  SIGCHLD, SIGCONT,   SIGFPE,  SIGHUP,
-    SIGILL,  SIGINT,  SIGKILL, SIGPIPE, SIGQUIT,   SIGSEGV, SIGSTOP,
-    SIGTERM, SIGTSTP, SIGTTIN, SIGTTOU, SIGUSR1,   SIGUSR2, SIGPOLL,
-    SIGPROF, SIGSYS,  SIGTRAP, SIGURG,  SIGVTALRM, SIGXCPU, SIGXFSZ,
+static const int signalValues[ARRAY_SIZE(signalNames)] = {
+  -1,
+  SIGHUP,
+  SIGINT,
+  SIGQUIT,
+  SIGILL,
+  SIGTRAP,
+  SIGABRT,
+  SIGBUS,
+  SIGFPE,
+  SIGKILL,
+  SIGUSR1,
+  SIGSEGV,
+  SIGUSR2,
+  SIGPIPE,
+  SIGALRM,
+  SIGTERM,
+  SIGCHLD,
+  SIGCONT,
+  SIGSTOP,
+  SIGTSTP,
+  SIGTTIN,
+  SIGTTOU,
+  SIGURG,
+  SIGXCPU,
+  SIGXFSZ,
+  SIGVTALRM,
+  SIGPROF,
+  SIGWINCH,
+  SIGIO,
+  SIGPWR,
+  SIGSYS,
 };
 
-static const size_t nvtop_num_signals = 28;
+static const size_t nvtop_num_signals = ARRAY_SIZE(signalNames) - 1;
 
 static void draw_kill_option(struct nvtop_interface *interface) {
   WINDOW *win = interface->process.option_window.option_win;
@@ -1313,7 +1376,7 @@ static void draw_kill_option(struct nvtop_interface *interface) {
     if (i == interface->process.option_window.selected_row) {
       wattron(win, COLOR_PAIR(cyan_color) | A_STANDOUT);
     }
-    wprintw(win, "%*d %s", 2, i, signalsName[i]);
+    wprintw(win, "%*d %s", 2, i, signalNames[i]);
     getyx(win, rows, cols);
 
     for (unsigned int j = cols; j < option_window_size; ++j)
@@ -1407,10 +1470,15 @@ static void draw_options(struct nvtop_interface *interface) {
 }
 
 static const char *option_selection_hidden[] = {
-    "Kill",
     "Sort",
+    "Kill",
     "Quit",
 };
+static const char *option_selection_hidden_num[] = {
+    "6",
+    "9",
+    "10",
+};
 
 static const char *option_selection_sort[][2] = {
     {"Enter", "Sort"},
@@ -1432,7 +1500,7 @@ static void draw_option_selection(struct nvtop_interface *interface) {
   switch (interface->process.option_window.state) {
   case nvtop_option_state_hidden:
     for (size_t i = 0; i < 3; ++i) {
-      wprintw(win, "F%zu", i + 1);
+      wprintw(win, "F%s", option_selection_hidden_num[i]);
       wattron(win, COLOR_PAIR(cyan_color) | A_STANDOUT);
       wprintw(win, "%s", option_selection_hidden[i]);
       for (size_t j = strlen(option_selection_hidden[i]);
@@ -1499,8 +1567,28 @@ static void update_retained_data(struct device_info *dev_info,
   }
 }
 
+static inline double *address_recent_left_old_right(size_t buffer_size,
+                                                    double buffer[buffer_size],
+                                                    size_t col_size, size_t col,
+                                                    size_t offset_in_column) {
+  return &buffer[col * col_size + offset_in_column];
+}
+
+static inline double *address_old_left_recent_right(size_t buffer_size,
+                                                    double buffer[buffer_size],
+                                                    size_t col_size, size_t col,
+                                                    size_t offset_in_column) {
+  return &buffer[buffer_size - (col + 1) * col_size + offset_in_column];
+}
+
+typedef double *(*plot_buffer_access_fn)(size_t, double *, size_t, size_t,
+                                         size_t);
+
 static void populate_plot_data_gpu_mem(struct nvtop_interface *interface,
                                        struct plot_window *plot) {
+  plot_buffer_access_fn access_fn = interface->plot_old_left_recent_right
+                                        ? address_old_left_recent_right
+                                        : address_recent_left_old_right;
   memset(plot->data, 0, plot->num_data * sizeof(double));
   unsigned num_cols = plot->type == plot_gpu_duo ? 4 : 2;
   unsigned upper_bound =
@@ -1514,23 +1602,23 @@ static void populate_plot_data_gpu_mem(struct nvtop_interface *interface,
     unsigned(*memdata)[interface->past_data.size_data_buffer] =
         (unsigned(*)[interface->past_data.size_data_buffer])
             interface->past_data.mem_util;
-    plot->data[i * num_cols] =
+    *access_fn(plot->num_data, plot->data, num_cols, i, 0) =
         gpudata[plot->gpu_ids[0]]
                [(interface->past_data.size_data_buffer +
                  interface->past_data.num_collected_data - i - 1) %
                 interface->past_data.size_data_buffer];
-    plot->data[i * num_cols + 1] =
+    *access_fn(plot->num_data, plot->data, num_cols, i, 1) =
         memdata[plot->gpu_ids[0]]
                [(interface->past_data.size_data_buffer +
                  interface->past_data.num_collected_data - i - 1) %
                 interface->past_data.size_data_buffer];
     if (plot->type == plot_gpu_duo) {
-      plot->data[i * num_cols + 2] =
+      *access_fn(plot->num_data, plot->data, num_cols, i, 2) =
           gpudata[plot->gpu_ids[1]]
                  [(interface->past_data.size_data_buffer +
                    interface->past_data.num_collected_data - i - 1) %
                   interface->past_data.size_data_buffer];
-      plot->data[i * num_cols + 3] =
+      *access_fn(plot->num_data, plot->data, num_cols, i, 3) =
           memdata[plot->gpu_ids[1]]
                  [(interface->past_data.size_data_buffer +
                    interface->past_data.num_collected_data - i - 1) %
@@ -1645,7 +1733,7 @@ static void option_do_kill(struct nvtop_interface *inter) {
   if (inter->process.option_window.selected_row == 0)
     return;
   pid_t pid = inter->process.all_process[inter->process.selected_row].pid;
-  int sig = signalsValues[inter->process.option_window.selected_row - 1];
+  int sig = signalValues[inter->process.option_window.selected_row];
   kill(pid, sig);
 }
 
@@ -1658,13 +1746,13 @@ static void option_change_sort(struct nvtop_interface *inter) {
 
 void interface_key(int keyId, struct nvtop_interface *inter) {
   switch (keyId) {
-  case KEY_F(1):
+  case KEY_F(9):
     if (inter->process.option_window.state == nvtop_option_state_hidden) {
       inter->process.option_window.state = nvtop_option_state_kill;
       inter->process.option_window.selected_row = 0;
     }
     break;
-  case KEY_F(2):
+  case KEY_F(6):
     if (inter->process.option_window.state == nvtop_option_state_hidden) {
       inter->process.option_window.state = nvtop_option_state_sort_by;
       inter->process.option_window.selected_row = 0;
